<?php
/**
 * @copyright    Copyright (C) 2009-2018 ACYBA SAS - All rights reserved..
 * @license        GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined('_JEXEC') or die('Restricted access');

if(!@include_once(rtrim(JPATH_ADMINISTRATOR, DIRECTORY_SEPARATOR).DIRECTORY_SEPARATOR.'components'.DIRECTORY_SEPARATOR.'com_acymailing'.DIRECTORY_SEPARATOR.'helpers'.DIRECTORY_SEPARATOR.'helper.php')){
	return;
};

if(!class_exists('cbTabHandler') || !method_exists($_PLUGINS, 'registerFunction') || class_exists('getAcyMailingTab')){
	//we should not be there!
	return;
}

$_PLUGINS->registerFunction('onUserActive', 'userActivated', 'getAcyMailingTab');
$_PLUGINS->registerFunction('onAfterDeleteUser', 'userDelete', 'getAcyMailingTab');
$_PLUGINS->registerFunction('onBeforeUserBlocking', 'onBeforeUserBlocking', 'getAcyMailingTab');

class getAcyMailingTab extends cbTabHandler{
	var $installed = true;
	var $errorMessage = 'This plugin can not work without the AcyMailing Component.<br/>Please download it from <a href="http://www.acyba.com">http://www.acyba.com</a> and install it.';

	function __construct(){
		if(!file_exists(rtrim(JPATH_ADMINISTRATOR, DIRECTORY_SEPARATOR).DIRECTORY_SEPARATOR.'components'.DIRECTORY_SEPARATOR.'com_acymailing')){
			$this->installed = false;
		}

		$this->cbTabHandler();
	}

	function getDisplayRegistration($tab, $user, $ui, $postdata){
		$return = array();

		$visibleLists = $this->params->get('lists', 'All');
		$config = acymailing_config();
		$hiddenLists = $config->get('autosub', 'None');
		$displayhtml = $this->params->get('display_html', 1);

		if($displayhtml){
			$htmlValue = acymailing_boolean("acymailingdata[user][html]", '', 1, acymailing_translation('HTML'), acymailing_translation('JOOMEXT_TEXT').'&nbsp;&nbsp;');
			$return[] = cbTabs::_createPseudoField($tab, acymailing_translation('RECEIVE'), $htmlValue, '', 'acymailingdataReceiveField', false);
		}

		$listsClass = acymailing_get('class.list');
		$allLists = $listsClass->getLists('listid');
		if(acymailing_level(1)){
			$allLists = $listsClass->onlyCurrentLanguage($allLists);
		}

		if(empty($allLists)) return $return;

		$visibleListsArray = array();
		$hiddenListsArray = array();

		//Load the lists...
		if(strpos($visibleLists, ',') OR is_numeric($visibleLists)){
			$allvisiblelists = explode(',', $visibleLists);
			foreach($allLists as $oneList){
				if($oneList->published AND in_array($oneList->listid, $allvisiblelists)){
					$visibleListsArray[] = $oneList->listid;
				}
			}
		}
		elseif(strtolower($visibleLists) == 'all'){
			foreach($allLists as $oneList){
				if($oneList->published){
					$visibleListsArray[] = $oneList->listid;
				}
			}
		}

		if(strpos($hiddenLists, ',') OR is_numeric($hiddenLists)){
			$hiddenListsArray = explode(',', $hiddenLists);
		}
		elseif(strtolower($hiddenLists) == 'all'){
			$visibleListsArray = array();
			foreach($allLists as $oneList){
				if($oneList->published){
					$hiddenListsArray[] = $oneList->listid;
				}
			}
		}

		if(!empty($visibleListsArray) AND !empty($hiddenListsArray)){
			$visibleListsArray = array_diff($visibleListsArray, $hiddenListsArray);
		}

		//Check lists...
		$checkedLists = $this->params->get('listschecked', 'All');
		if(strtolower($checkedLists) == 'all'){
			$checkedListsArray = $visibleListsArray;
		}
		elseif(strpos($checkedLists, ',') OR is_numeric($checkedLists)){
			$checkedListsArray = explode(',', $checkedLists);
		}
		else{
			$checkedListsArray = array();
		}

		$label = $this->params->get('subcaption');
		if(empty($label)) $label = acymailing_translation('SUBSCRIPTION');
		$hiddenLists = implode(',', $hiddenListsArray);
		if(!empty($visibleListsArray)){
			$listsHtml = '<table style="border:0px;">';
			foreach($visibleListsArray as $oneList){
				$check = in_array($oneList, $checkedListsArray) ? 'checked="checked"' : '';
				$name = $this->params->get('addoverlay', false) ? acymailing_tooltip($allLists[$oneList]->description, $allLists[$oneList]->name, '', $allLists[$oneList]->name) : $allLists[$oneList]->name;
				$listsHtml .= '<tr style="border:0px;"><td style="border:0px;"><input type="checkbox" class="acymailing_checkbox" id="acy_'.$oneList.'" name="acymailingdata[subscription][]" '.$check.' value="'.$oneList.'"/></td><td style="border:0px;"><label for="acy_'.$oneList.'">'.$name.'</label></td></tr>';
			}
			$listsHtml .= '</table><input type="hidden" name="visibleLists" value="'.implode(',', $visibleListsArray).'" />';
			$return[] = cbTabs::_createPseudoField($tab, $label, $listsHtml, '', 'acymailingdataLists', false);
		}

		return $return;
	}

	function getDisplayTab($tab, $user, $ui){

		// Compatibility with CB profile pro
		if(file_exists(JPATH_SITE.DS.'components'.DS.'com_cbprofilepro')){
			if(!empty($_REQUEST['task']) && $_REQUEST['task'] == 'userdetails') return $this->getEditTab($tab, $user, $ui); // Edit page
			if(empty($user->user_id) && empty($user->id)) return $this->getDisplayRegistration($tab, $user, $ui, ''); // Registration page
		}

		$currentUserid = acymailing_currentUserId();
		if(empty($currentUserid) || $currentUserid != $user->user_id) return;

		$userClass = acymailing_get('class.subscriber');
		$joomUser = $userClass->get($user->email);

		if(empty($joomUser->subid)) return;

		$config = acymailing_config();
		$cssFrontend = $config->get('css_frontend', 'default');
		if(!empty($cssFrontend)){
			acymailing_addStyle(false, ACYMAILING_CSS.'component_'.$cssFrontend.'.css?v='.filemtime(ACYMAILING_MEDIA.'css'.DS.'component_'.$cssFrontend.'.css'));
		}

		// Checking if user is confirmed in Acymailing
		if($joomUser->confirmed == 0 && $config->get('require_confirmation') == 1){
			$itemId = $config->get('itemid', 0);
			$item = empty($itemId) ? '' : '&Itemid='.$itemId;
			$myLink = acymailing_frontendLink('index.php?subid='.$joomUser->subid.'&option=com_acymailing&ctrl=user&task=confirm&key='.urlencode($joomUser->key).$item, false);
			acymailing_display(acymailing_translation('ACY_SUB_NOT_CONFIRMED').' '.'<a target="_blank" href="'.$myLink.'">'.acymailing_translation('CONFIRMATION_CLICK').'</a>', 'warning');
		}

		$listsClass = acymailing_get('class.list');
		$allLists = $userClass->getSubscription($joomUser->subid, 'listid');

		if(acymailing_level(1)){
			$allLists = $listsClass->onlyCurrentLanguage($allLists);
		}

		if(acymailing_level(3)){
			foreach($allLists as $listid => $oneList){
				if(!$allLists[$listid]->published) continue;
				if(!acymailing_isAllowed($oneList->access_sub)){
					$allLists[$listid]->published = false;
					continue;
				}
			}
		}

		$lists = $this->params->get('listsprofile', 'All');

		$visibleListsArray = array();
		if(strpos($lists, ',') OR is_numeric($lists)){
			$allvisiblelists = explode(',', $lists);
			foreach($allLists as $oneList){
				if($oneList->published AND in_array($oneList->listid, $allvisiblelists)){
					$visibleListsArray[] = $oneList->listid;
				}
			}
		}
		elseif(strtolower($lists) == 'all'){
			foreach($allLists as $oneList){
				if($oneList->published){
					$visibleListsArray[] = $oneList->listid;
				}
			}
		}

		foreach($visibleListsArray as $i => $oneListid){
			if($allLists[$oneListid]->status <= 0) unset($visibleListsArray[$i]);
		}

		if(empty($visibleListsArray)) return;

		$return = '';
		$introText = $this->params->get('introtext');
		if(!empty($introText)){
			$return .= '<div class="acymailing_introtext" >'.$introText.'</div>';
		}

		if($this->params->get('display_htmlprofile', 1)){
			$return .= '<table style="margin-bottom: 20px;"><tr><td class="titleCell">'.acymailing_translation('RECEIVE').': </td><td class="fieldCell">'.(empty($joomUser->subid) || $joomUser->html == 1 ? acymailing_translation('HTML_VERSION') : acymailing_translation('TEXT_VERSION')).'</td></tr></table>';
		}

		$return .= '<table class="acycbsubscription">';
		$k = 0;
		foreach($visibleListsArray as $oneListid){
			$return .= '<tr class="row'.$k.'"><td valign="top"><div class="list_name">'.$allLists[$oneListid]->name.'</div><div class="list_description">'.$allLists[$oneListid]->description.'</div></td></tr>';
			$k = 1 - $k;
		}

		$return .= '</table>';

		return $return;
	}

	function saveRegistrationTab($tab, &$user, $ui, $postdata){
		if((empty($user->id) && empty($user->user_id)) OR !$this->installed) return;

		$subscriberClass = acymailing_get('class.subscriber');
		$subscriberClass->checkVisitor = false;
		$subscriberClass->sendConf = false;
		$subscriber = $subscriberClass->get($user->email);
		if(empty($subscriber)) $subscriber = new stdClass();

		$subscriber->email = $user->email;
		if(!empty($user->name)) $subscriber->name = $user->name;
		if(!empty($user->user_id)) $subscriber->userid = $user->user_id;
		elseif(!empty($user->id)) $subscriber->userid = $user->id;
		if(isset($user->confirmed)) $subscriber->confirmed = $user->confirmed;
		if(!empty($user->block)) $subscriber->enabled = 0;
		if($this->params->get('enabled', 0) == 1) $subscriber->enabled = 1;

		if(!empty($postdata['acymailingdata']['user'])){
			$subscriberClass->checkFields($postdata['acymailingdata']['user'], $subscriber);
		}

		$this->assocFields($user, $subscriber);

		$subscriber->subid = $subscriberClass->save($subscriber);

		//Subscription...
		$config = acymailing_config();
		$statusAdd = (empty($user->confirmed) AND $config->get('require_confirmation', false)) ? 2 : 1;

		$currentSubscription = $subscriberClass->getSubscriptionStatus($subscriber->subid);

		$hiddenLists = $config->get('autosub', 'None');
		$listsClass = acymailing_get('class.list');
		$allLists = $listsClass->getLists('listid');
		if(acymailing_level(1)){
			$allLists = $listsClass->onlyCurrentLanguage($allLists);
		}

		if(strpos($hiddenLists, ',') OR is_numeric($hiddenLists)){
			$hiddenListsArray = explode(',', $hiddenLists);
		}
		elseif(strtolower($hiddenLists) == 'all'){
			$hiddenListsArray = array_keys($allLists);
		}

		$addlists = array();
		if(!empty($hiddenListsArray)){
			foreach($hiddenListsArray as $idOneList){
				if(!empty($allLists[$idOneList]->published) && empty($currentSubscription[$idOneList]->status)) $addlists[$statusAdd][] = $idOneList;
			}
		}

		if(!empty($postdata['acymailingdata']['subscription'])){
			foreach($postdata['acymailingdata']['subscription'] as $idOneList){
				if(!empty($allLists[$idOneList]->published) && empty($currentSubscription[$idOneList]->status)) $addlists[$statusAdd][] = $idOneList;
			}
		}

		$listsubClass = acymailing_get('class.listsub');
		if(!empty($user->gid)) $listsubClass->gid = $user->gid;
		if(!empty($addlists)){
			$listsubClass->addSubscription($subscriber->subid, $addlists);
		}

		if($this->params->get('updateonregister', 0) && !empty($currentSubscription)){
			$updateLists = array();
			//We should remove the user subscription if he unchecked some lists...
			//Is the user an existing one? If not, we just don't care...
			$allvisiblelists = acymailing_getVar('string', 'visibleLists');
			$allvisiblelistsArray = explode(',', $allvisiblelists);
			foreach($allvisiblelistsArray as $oneListId){
				//The user is not subscribed to the list anyway...
				if(empty($currentSubscription[intval($oneListId)]->status)) continue;

				//this list was checked!
				if(!empty($postdata['acymailingdata']['subscription']) && in_array(intval($oneListId), $postdata['acymailingdata']['subscription'])){
					//List checked!
					//The user is already subscribed to that list
					if($currentSubscription[intval($oneListId)]->status > 0) continue;

					$updateLists[$statusAdd][] = intval($oneListId);
				}
				else{
					//List not checked!
					//The user is already unsubscribed from that list...
					if($currentSubscription[intval($oneListId)]->status < 0) continue;

					$updateLists[-1][] = intval($oneListId);
				}
			}

			if(!empty($updateLists)){
				$listsubClass->updateSubscription($subscriber->subid, $updateLists);
			}
		}

		return;
	}

	function userDelete($user, $success){
		if(!$this->installed){
			return $this->errorMessage;
		}

		if(!$success) return;

		$userClass = acymailing_get('class.subscriber');
		$subid = $userClass->subid($user->email);
		if(!empty($subid)){
			$userClass->delete($subid);
		}
	}

	function userActivated($user, $success){
		if(!$this->installed){
			return $this->errorMessage;
		}

		if(!$success) return;

		$userClass = acymailing_get('class.subscriber');
		$subid = $userClass->subid($user->email);
		if(!empty($subid)){
			if(empty($user->block)){
				acymailing_query('UPDATE `#__acymailing_subscriber` SET `enabled` = 1 WHERE `subid` = '.$subid.' LIMIT 1');
			}
			$userClass->confirmSubscription($subid);
		}

		return;
	}

	function onBeforeUserBlocking($user, $block){
		acymailing_query('UPDATE `#__acymailing_subscriber` SET `enabled` = '.(1 - intval($block)).' WHERE `userid` = '.intval($user->id).' LIMIT 1');
	}

	function getEditTab($tab, $user, $ui){
		if(!$this->installed){
			return $this->errorMessage;
		}

		$config = acymailing_config();
		$cssFrontend = $config->get('css_frontend', 'default');
		if(!empty($cssFrontend)){
			acymailing_addStyle(false, ACYMAILING_CSS.'component_'.$cssFrontend.'.css?v='.filemtime(ACYMAILING_MEDIA.'css'.DS.'component_'.$cssFrontend.'.css'));
		}

		$return = '';

		$lists = $this->params->get('listsprofile', 'All');
		$displayhtml = $this->params->get('display_htmlprofile', 1);

		$userClass = acymailing_get('class.subscriber');
		$joomUser = $userClass->get($user->email);

		if(empty($joomUser->subid)){
			//User not inserted?? How is that possible...? well, if it's a new!
		}
		elseif(!empty($user->id) AND (int) $joomUser->userid != (int) $user->id){
			//Update the field so that it's linked to this user so during the update we can keep the same user.
			//Let's update the user Id...
			acymailing_query('UPDATE '.acymailing_table('subscriber').' SET userid = '.(int) $user->id.' WHERE subid = '.(int) $joomUser->subid.' LIMIT 1');
		}

		if(!empty($joomUser)){
			// Checking if user is confirmed in Acymailing
			if($joomUser->confirmed == 0 && $config->get('require_confirmation') == 1){
				$itemId = $config->get('itemid', 0);
				$item = empty($itemId) ? '' : '&Itemid='.$itemId;
				$myLink = acymailing_frontendLink('index.php?subid='.$joomUser->subid.'&option=com_acymailing&ctrl=user&task=confirm&key='.urlencode($joomUser->key).$item, false);
				acymailing_display(acymailing_translation('ACY_SUB_NOT_CONFIRMED').' '.'<a target="_blank" href="'.$myLink.'">'.acymailing_translation('CONFIRMATION_CLICK').'</a>', 'warning');
			}
		}

		$listsClass = acymailing_get('class.list');
		if(!empty($joomUser->subid)){
			$allLists = $userClass->getSubscription($joomUser->subid, 'listid');
		}
		else{
			$allLists = $listsClass->getLists('listid');
		}

		if(!acymailing_isAdmin() AND acymailing_level(1)){
			$allLists = $listsClass->onlyCurrentLanguage($allLists);
		}

		if(!acymailing_isAdmin() AND acymailing_level(3)){
			foreach($allLists as $listid => $oneList){
				if(!$allLists[$listid]->published) continue;
				if(!acymailing_isAllowed($oneList->access_sub)){
					$allLists[$listid]->published = false;
					continue;
				}
			}
		}

		//If we come from the Admin interface
		if(acymailing_isAdmin()){
			$lists = $this->params->get('listsprofileback', 'All');
			$visibleListsArray = array();
			if(strpos($lists, ',') OR is_numeric($lists)){
				$allvisiblelists = explode(',', $lists);
				foreach($allLists as $oneList){
					if(in_array($oneList->listid, $allvisiblelists)){
						$visibleListsArray[] = $oneList->listid;
					}
				}
			}
			elseif(strtolower($lists) == 'all'){
				foreach($allLists as $oneList){
					$visibleListsArray[] = $oneList->listid;
				}
			}

		} else {
			$visibleListsArray = array();
			if(strpos($lists, ',') OR is_numeric($lists)){
				$allvisiblelists = explode(',', $lists);
				foreach($allLists as $oneList){
					if($oneList->published AND in_array($oneList->listid, $allvisiblelists)){
						$visibleListsArray[] = $oneList->listid;
					}
				}
			}
			elseif(strtolower($lists) == 'all'){
				foreach($allLists as $oneList){
					if($oneList->published){
						$visibleListsArray[] = $oneList->listid;
					}
				}
			}
		}

		if($displayhtml){
			//Add the receive text/html option
			$return .= '<table><tr><td class="titleCell">'.acymailing_translation('RECEIVE').'</td><td class="fieldCell">'.acymailing_boolean("acymailingdata[user][html]", '', empty($joomUser->subid) ? 1 : $joomUser->html, acymailing_translation('HTML'), acymailing_translation('JOOMEXT_TEXT').'&nbsp;&nbsp;').'</td></tr></table>';
		}

		if(!empty($visibleListsArray)){
			if(acymailing_isAdmin()){
				$status = acymailing_get('type.status');
			}
			else{
				$status = acymailing_get('type.festatus');
			}


			$return .= '<table class="acycbsubscription"><tr><th>'.acymailing_translation('SUBSCRIPTION').'</th><th>'.acymailing_translation('LIST').'</th></tr>';
			$k = 0;
			foreach($visibleListsArray as $oneListid){
				$return .= '<tr class="row'.$k.'"><td align="center" valign="top" nowrap="nowrap" >'.$status->display("acymailingdata[listsub][".$oneListid."][status]", @$allLists[$oneListid]->status).'</td>';
				$return .= '<td valign="top"><div class="list_name">'.$allLists[$oneListid]->name.'</div><div class="list_description">'.$allLists[$oneListid]->description.'</div></td></tr>';
				$k = 1 - $k;
			}

			$return .= '</table>';
		}

		return $return;
	}

	//Match CB fields to Acy ones...
	private function assocFields(&$user, &$subscriber){
		$params = $this->params->get('assocfields');
		if(empty($params)) return;

		$rules = explode(';', str_replace("\n", ';', trim($params)));
		if(empty($rules)) return;

		foreach($rules as $oneRule){
			if(!strpos($oneRule, '=')) continue;
			list($acyField, $cbRule) = explode('=', trim($oneRule));
			$cbFields = explode('+', trim($cbRule));

			//We reset it... always
			unset($subscriber->{$acyField});

			foreach($cbFields as $cbField){
				if(!isset($user->get('_comprofilerUser')->{$cbField})) continue;

				//Add a space in case of concatenation... or we init it
				if(empty($subscriber->{$acyField})){
					$subscriber->{$acyField} = '';
				}
				else{
					$subscriber->{$acyField} .= ' ';
				}

				$subscriber->{$acyField} .= $user->get('_comprofilerUser')->{$cbField};
			}
		}
	}

	function saveEditTab($tab, &$user, $ui, $postdata){
		$subscriberClass = acymailing_get('class.subscriber');
		$subscriberClass->triggerFilterBE = true;
		$subscriber = new stdClass();
		if(!empty($user->id)) $subscriber->subid = $subscriberClass->subid($user->id);
		if(!empty($subscriber->subid)){
			$currentSubid = $subscriberClass->subid($user->email);
			if(!empty($currentSubid) && $subscriber->subid != $currentSubid){
				//The user already exists and has a different subid... that will block the update.
				//We will delete that user.
				$subscriberClass->delete($currentSubid);
			}
		}

		//We use the user->id to update the infos as we are sure it does exists.
		if(!empty($postdata['acymailingdata']['user'])){
			//Check the infos...
			$subscriberClass->checkFields($postdata['acymailingdata']['user'], $subscriber);
		}

		if(!empty($user->name)) $subscriber->name = $user->name;
		if(!empty($user->email)) $subscriber->email = $user->email;
		$subscriber->enabled = empty($user->block) ? 1 : 0;
		$subscriber->confirmed = $user->confirmed;
		$subscriber->userid = $user->id;

		$this->assocFields($user, $subscriber);

		$subscriber->subid = $subscriberClass->save($subscriber);

		if(empty($subscriber->subid)) return;

		if(!empty($postdata['acymailingdata']['listsub'])){
			$subscriberClass->saveSubscription($subscriber->subid, $postdata['acymailingdata']['listsub']);
		}
	}

	function help($name, $value, $control_name){
		if(!$this->installed) return $this->errorMessage;

		$config = acymailing_config();
		$level = $config->get('level');

		$link = ACYMAILING_HELPURL.'cbplugin'.'&level='.$level;
		$text = acymailing_popup($link, '<button onclick="return false">'.acymailing_translation('ACY_HELP').'</button>', '', 800, 500, '', ' style="display:inline-block;overflow:inherit;position:inherit;" ');

		return $text;
	}

	function lists($name, $value, $control_name){
		if(!$this->installed) return $this->errorMessage;

		$link = 'index.php?option=com_acymailing&amp;tmpl=component&amp;ctrl=chooselist&amp;task='.$name.'&amp;control='.$control_name.'&amp;values='.$value;
		$text = '<input class="inputbox" id="'.$control_name.$name.'" name="'.$control_name.'['.$name.']" type="text" size="20" value="'.$value.'">';
		$text .= acymailing_popup($link, '<button onclick="return false">'.acymailing_translation('Select').'</button>', '', 650, 375, 'link'.$control_name.$name, ' style="display:inline-block;overflow:inherit;position:inherit;" ');

		return $text;
	}
}

